"""
  >>> from megrok.layout import ILayout
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> mongo = Dummy()
  >>> mylayout = getMultiAdapter((request, mongo), ILayout)
  >>> mylayout.static
  <grokcore.view.components.DirectoryResource object at ...>
  >>> mylayout.static['empty.js']
  <zope.app.publisher.browser.fileresource.FileResource object at ...>
"""

import grokcore.component as grok
from megrok.layout import Layout


class Dummy(grok.Context):
    pass


class LayoutWithResources(Layout):

    def render(self):
        return ""

def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite  



"""
  >>> from zope.app.testing.functional import getRootFolder
  >>> getRootFolder()["a"] = A()
  >>> getRootFolder()["b"] = B()
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.handleErrors = False 
  
  >>> browser.open("http://localhost/++skin++mydefaultskin/a/@@myview") 
  >>> print browser.contents
  <div> A Layout </div>

  >>> browser.open("http://localhost/++skin++myskin/a/@@myview") 
  >>> print browser.contents
  <div> A2 Layout </div>
  
  >>> browser.open("http://localhost/++skin++myskin/b/@@myviewb")
  >>> print browser.contents
  <div> B Layout </div>
"""

import grokcore.component as grok
from grokcore.view import layer, skin

from zope import interface
from megrok.layout import Layout, Page 

from grokcore.view import IDefaultBrowserLayer

class IMyDefaultLayer(IDefaultBrowserLayer):
    pass


class MyDefaultSkin(IMyDefaultLayer):
    skin('mydefaultskin')


layer(IMyDefaultLayer)



class MySkinLayer(IDefaultBrowserLayer):
    pass


class MySkin(MySkinLayer):
    skin('myskin')


class A(grok.Context):
    pass


class B(grok.Context):
    pass


class ALayout(Layout):
    grok.context(A)

    def render(self):
	return "<div> A Layout </div>"


class A2Layout(Layout):
    grok.context(A)
    layer(MySkinLayer)

    def render(self):
	return "<div> A2 Layout </div>"


class BLayout(Layout):
    grok.context(B)
    layer(MySkinLayer)

    def render(self):
	return "<div> B Layout </div>"


class MyView(Page):
    grok.context(interface.Interface)

    def render(self):
        return "MYVIEW"


class MyViewB(Page):
    grok.context(interface.Interface)
    layer(MySkinLayer)

    def render(self):
        return "MYVIEW"


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite  



"""
  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  ...
  ConfigurationExecutionError: <class 'martian.error.GrokError'>: View <class 'megrok.layout.tests.test_norenderortemplatelayout.MyLayout'> has no associated template or 'render' method.
    in:
  <BLANKLINE>
"""

import grokcore.component as grok
from grokcore.view import View
from megrok.layout import Layout
from zope.interface import Interface


class MyLayout(Layout):
    grok.context(Interface)


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite



"""
  >>> from megrok.layout import ILayout
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> cow = Cow()
  >>> mylayout = getMultiAdapter((request, cow), ILayout)
  >>> myview = getMultiAdapter((cow, request), name='myview')


  >>> print myview()
  <html>
   <body>
     <div class="layout"><p> My nice Content </p></div>
   </body>
  </html>

 
  >>> myview
  <megrok.layout.ftests.test_page.MyView object at ...>
  >>> myview.layout
  <megrok.layout.ftests.test_page.Master object at ...>
  >>> print myview.content
  <p> My nice Content </p>

"""
import grokcore.component as grok
from grokcore.view import templatedir

from zope import interface
from megrok.layout import Layout, Page

templatedir('templates')


class Cow(grok.Context):
    pass


class Master(Layout):
    grok.name('master')
    grok.context(Cow)


class MyView(Page):
    grok.context(interface.Interface)

    def render(self):
	return "<p> My nice Content </p>"


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite  



import os.path                                                                                                               
import megrok.layout
from zope.app.testing.functional import ZCMLLayer

ftesting_zcml = os.path.join(os.path.dirname(megrok.layout.__file__),
                             'ftesting.zcml')
FunctionalLayer = ZCMLLayer(ftesting_zcml, __name__, 'FunctionalLayer',
                            allow_teardown=True)




import unittest
import doctest
from zope.testing import cleanup
from zope.testing import module
import zope.component.eventtesting
from zope import component
from megrok import layout

    
def moduleSetUp(test):
    module.setUp(test, '__main__')
    
def moduleTearDown(test):    
    module.tearDown(test)
    cleanup.cleanUp()
    
def zopeSetUp(test):
    zope.component.eventtesting.setUp(test)

def zopeTearDown(test):
    cleanup.cleanUp()
    
def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    globs = {}
    suite = unittest.TestSuite()
    
    suite.addTest(
        doctest.DocFileSuite(
            '../README.txt',
            optionflags=optionflags,
            setUp=moduleSetUp,
            tearDown=moduleTearDown,
            globs=globs)
        )

    return suite



from zope.configuration.config import ConfigurationMachine
from grokcore.component import zcml

def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok('grokcore.component.meta', config)
    zcml.do_grok('grokcore.security.meta', config)
    zcml.do_grok('grokcore.view.meta', config)
    zcml.do_grok('grokcore.view.templatereg', config)
    zcml.do_grok(module_name, config)
    config.execute_actions()



"""
  >>> from megrok.layout import ILayout
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> mammoth = Mammoth()
  >>> mylayout = getMultiAdapter((request, mammoth), ILayout)
  >>> ILayout.providedBy(mylayout)
  True

  >>> mylayout.context
  <megrok.layout.ftests.test_layout.Mammoth object at ...>

  >>> mylayout.render()
  '<div> MyLayout </div>'

  >>> elephant = Elephant()
  >>> mycontextlayout = getMultiAdapter((request, elephant), ILayout)
  >>> mycontextlayout.render()
  '<div> MyContextLayout </div>'
"""

import grokcore.component as grok

from zope import interface
from megrok.layout import Layout 


class Mammoth(grok.Context):
    pass


class Elephant(grok.Context):
    pass


class MyLayout(Layout):
    grok.context(interface.Interface)

    def render(self):
	return "<div> MyLayout </div>"


class MyContextLayout(Layout):
    grok.context(Elephant)

    def render(self):
	return "<div> MyContextLayout </div>"


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite  



# -*- coding: utf-8 -*-

import zope.component
import grokcore.view
import grokcore.formlib
import grokcore.component as grok
from zope.interface import Interface
from zope.publisher.publish import mapply
from megrok.layout import IPage, ILayout


class Layout(object):
    """A layout object.
    """
    grok.baseclass()
    grok.implements(ILayout)

    def __init__(self, request, context):
        self.context = context
        self.request = request
        self.view = None

        if getattr(self, 'module_info', None) is not None:
            self.static = zope.component.queryAdapter(
                self.request, Interface,
                name=self.module_info.package_dotted_name
                )
        else:
            self.static = None

    def default_namespace(self):
        namespace = {}
        namespace['view'] = self.view
        namespace['layout'] = self
        namespace['static'] = self.static
        namespace['context'] = self.context
        namespace['request'] = self.request
        return namespace

    def namespace(self):
        return {}

    def update(self):
        pass

    @property
    def response(self):
        return self.request.response

    def _render_template(self):
        return self.template.render(self)

    def render(self):
        return self._render_template()

    render.base_method = True

    def __call__(self, view):
        self.view = view
        self.update()
        return self.render()


class Page(grokcore.view.View):
    """A view class.
    """
    grok.baseclass()
    grok.implements(IPage)

    template = None

    def __init__(self, context, request):
        super(Page, self).__init__(context, request)
        self.layout = None

    def default_namespace(self):
        namespace = super(Page, self).default_namespace()
        namespace['layout'] = self.layout
        return namespace

    def render(self):
        return self._render_template()

    render.base_method = True

    @property
    def content(self):
        template = getattr(self, 'template', None)
        if template is not None:
            return self._render_template()
        return mapply(self.render, (), self.request)

    def __call__(self):
        mapply(self.update, (), self.request)
        if self.request.response.getStatus() in (302, 303):
            # A redirect was triggered somewhere in update().  Don't
            # continue rendering the template or doing anything else.
            return
        self.layout = zope.component.getMultiAdapter(
            (self.request, self.context), ILayout)
        return self.layout(self)


class Form(grokcore.formlib.Form):
    """A form class.
    """
    grok.baseclass()
    
    def __init__(self, context, request):
        super(Form, self).__init__(context, request)
        self.layout = None

    def default_namespace(self):
        namespace = super(Form, self).default_namespace()
        namespace['layout'] = self.layout
        return namespace

    @property
    def content(self):
        template = getattr(self, 'template', None)
        if template is not None:
            return self._render_template()
        return mapply(self.render, (), self.request)
    
    def __call__(self):
        """Calls update and returns the layout template which calls render.
        """
        mapply(self.update, (), self.request)
        if self.request.response.getStatus() in (302, 303):
            # A redirect was triggered somewhere in update().  Don't
            # continue rendering the template or doing anything else.
            return
        
        self.update_form()
        if self.request.response.getStatus() in (302, 303):
            return
        
        self.layout = zope.component.getMultiAdapter(
            (self.request, self.context), ILayout)
        return self.layout(self)



"""
  >>> from zope.app.testing.functional import getRootFolder
  >>> getRootFolder()["one"] = One()
  >>> getRootFolder()["two"] = Two()
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.handleErrors = False 
  
  >>> browser.open("http://localhost/++skin++askin/one/@@myview") 
  >>> print browser.contents
  <div> Layout A for context One </div>

  >>> browser.open("http://localhost/++skin++askin/two/@@myview") 
  >>> print browser.contents
  <div> Layout A for context Two </div>

  >>> browser.open("http://localhost/++skin++bskin/one/@@myview") 
  >>> print browser.contents
  <div> Layout B for context One </div>

  >>> browser.open("http://localhost/++skin++bskin/two/@@myview") 
  >>> print browser.contents
  <div> Layout B for context Two </div>
"""

import grokcore.view as grok
from grokcore.view import layer, skin

from zope import interface
from megrok.layout import Layout, Page 

from grokcore.view import IDefaultBrowserLayer

class IALayer(IDefaultBrowserLayer):
    pass


class IBLayer(IALayer):
    pass


class ASkin(IALayer):
    skin('askin')


class BSkin(IBLayer):
    skin('bskin')


class One(grok.Context):
    pass


class Two(One):
    pass


class AOneLayout(Layout):
    grok.context(One)
    grok.layer(IALayer)

    def render(self):
	return "<div> Layout A for context One </div>"

class ATwoLayout(Layout):
    grok.context(Two)
    grok.layer(IALayer)

    def render(self):
	return "<div> Layout A for context Two </div>"

class BOneLayout(Layout):
    grok.context(One)
    grok.layer(IBLayer)

    def render(self):
	return "<div> Layout B for context One </div>"

class BTwoLayout(Layout):
    grok.context(Two)
    grok.layer(IBLayer)

    def render(self):
	return "<div> Layout B for context Two </div>"


class MyView(Page):
    grok.context(interface.Interface)
    grok.layer(IALayer)

    def render(self):
        return "MyView on IALayouer"


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite  



# -*- coding: utf-8 -*-

import martian
import zope.component
import grokcore.component
from megrok.layout import ILayout, Layout
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class LayoutGrokker(martian.ClassGrokker):

    martian.component(Layout)
    martian.directive(grokcore.component.context)
    martian.directive(grokcore.view.layer, default=IDefaultBrowserLayer)

    def grok(self, name, factory, module_info, **kw):
        factory.module_info = module_info
        return super(LayoutGrokker, self).grok(name, factory, module_info, **kw)

    def execute(self, factory, config, context, layer, **kw):
        # find templates
        templates = factory.module_info.getAnnotation('grok.templates', None)
        if templates is not None:
            config.action(
                discriminator=None,
                callable=self.checkTemplates,
                args=(templates, factory.module_info, factory)
                )

        adapts = (layer, context)
        config.action(
            discriminator=('adapter', adapts, ILayout),
            callable=zope.component.provideAdapter,
            args=(factory, adapts, ILayout),
            )
        return True

    def checkTemplates(self, templates, module_info, factory):

        def has_render(factory):
            render = getattr(factory, 'render', None)
            base_method = getattr(render, 'base_method', False)
            return render and not base_method

        def has_no_render(factory):
            render = getattr(factory, 'render', None)
            base_method = getattr(render, 'base_method', False)
            return render is None or base_method  
        templates.checkTemplates(module_info, factory, 'view',
                                 has_render, has_no_render)



"""
  >>> grok.testing.grok(__name__) 
  Traceback (most recent call last):
  ...
  ConfigurationConflictError: Conflicting configuration actions
     For: ('adapter', (<InterfaceClass zope.publisher.interfaces.browser.IDefaultBrowserLayer>, <InterfaceClass zope.interface.Interface>), <InterfaceClass megrok.layout.interfaces.ILayout>)
"""

import grokcore.component as grok
from megrok.layout import Layout
from zope.interface import Interface


class MyLayout(Layout):
    grok.context(Interface)


class MyOtherLayout(Layout):
    grok.context(Interface)


def test_suite():
    from zope.testing import doctest
    from megrok.layout.ftests import FunctionalLayer
    suite = doctest.DocTestSuite(
        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS
        )
    suite.layer = FunctionalLayer
    return suite



from setuptools import setup, find_packages
import os

long_description = open('README.txt').read() + \
                    '\n\n' + \
                    open(os.path.join('docs', 'HISTORY.txt')).read()



setup(name='megrok.layout',
      version='0.6',
      description="A layout component package for zope3 and Grok.",
      long_description = long_description,
      classifiers=[
          "Framework :: Zope3",
          "Programming Language :: Python",
          "Programming Language :: Zope",
          "Intended Audience :: Developers",
          "Development Status :: 4 - Beta",
          "Topic :: Software Development :: Libraries :: Python Modules",
          ],
      keywords='grok layout zope3 pagelet theming',
      author='Souheil Chelfouh',
      author_email='trollfot@gmail.com',
      url='http://pypi.python.org/pypi/megrok.layout',
      license='GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages=['megrok'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
	  'grokcore.component',
	  'grokcore.view',
	  'grokcore.formlib',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-

from zope.interface import Interface


class ILayout(Interface):
    """Layout code.
    """

class IPage(Interface):
    """A template using a layout to render itself.
    """
